/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package gontogle;

import AutoAnnotation.AutomaticAnnotatorCmd;
import CmdLine.IndexInterface;
import CmdLine.ProtegeInterface;
import Ontology.AnnotationObj;
import Ontology.Ontology;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 *
 * @author Cyan
 */
public class Annotate {

    private static int threshold = 25; // by default set threshold to 25%
    private static String gi2moAnnotationProperty = "gi2mo:hasCharacteristics";
    private static String gi2moAnnotationPropertyPrefix = "http://gi2mo.org/Category/IKMO/";
    private static String gi2moTextProperty = "gi2mo:content";

    /** @param filePath the name of the file to open. Not sure if it can accept URLs or just filenames. Path handling could be better, and buffer sizes are hardcoded
    */
    private static String readFileAsString(String filePath)
    throws java.io.IOException{
        StringBuffer fileData = new StringBuffer(1000);
        BufferedReader reader = new BufferedReader(
                new FileReader(filePath));
        char[] buf = new char[1024];
        int numRead=0;
        while((numRead=reader.read(buf)) != -1){
            String readData = String.valueOf(buf, 0, numRead);
            fileData.append(readData);
            buf = new char[1024];
        }
        reader.close();
        return fileData.toString();
    }

    private static void writeFileAsTxt(String fileName, String text)
    {
        try {
            File file = new File(fileName);
            //File file = new File("test.txt");
            boolean exist = file.createNewFile();
            if (exist) {
                try {
                    // Create file
                    FileWriter fstream = new FileWriter(file); //"out.txt");
                    BufferedWriter out = new BufferedWriter(fstream);
                    out.write(text);
                    //Close the output stream
                    //out.flush();
                    out.close();
                    fstream.close();
                } catch (Exception e) {
                    //Catch exception if any
                    System.err.println("Error: " + e.getMessage());
                }
            }
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }
    
     public static HashMap<String,ArrayList<String>> parseFilelist(String filename) {

      HashMap<String,ArrayList<String>> files = new HashMap();
      
      try {
      File file = new File(filename);
      DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
      DocumentBuilder db = dbf.newDocumentBuilder();
      Document doc = db.parse(file);
      doc.getDocumentElement().normalize();
      NodeList nodeLst = doc.getElementsByTagName("file");

      for (int s = 0; s < nodeLst.getLength(); s++) {
        String fileName =  null;
        ArrayList<String> annotations = new ArrayList<String>();

        Node fstNode = nodeLst.item(s);

        if (fstNode.getNodeType() == Node.ELEMENT_NODE) {

          Element fstElmnt = (Element) fstNode;
          //get filename
          NodeList fstNmElmntLst = fstElmnt.getElementsByTagName("filename");
          Element fstNmElmnt = (Element) fstNmElmntLst.item(0);
          NodeList fstNm = fstNmElmnt.getChildNodes();
          //System.out.println("File Name : "  + ((Node) fstNm.item(0)).getNodeValue());
          fileName = ((Node) fstNm.item(0)).getNodeValue();
          //get annotations
          NodeList lstNmElmntLst = fstElmnt.getElementsByTagName("annotations");
          if (lstNmElmntLst.getLength()>0)
          {
              Element annoatationsNode = (Element) lstNmElmntLst.item(0);
              NodeList annoatationsNodeList = annoatationsNode.getElementsByTagName("name");
                for (int k = 0; k < annoatationsNodeList.getLength(); k++) {
                  Node annoatationNode = annoatationsNodeList.item(k);
                  if (annoatationNode.getNodeType() == Node.ELEMENT_NODE) {
                       Element annotation = (Element) annoatationNode;
                       annotations.add(annoatationNode.getTextContent());
                       //System.out.println("test:"+ annoatationNode.getTextContent());
                  }
              }
          }
        }
        files.put(fileName, annotations);
      }
      } catch (Exception e) {
        e.printStackTrace();
        return null;
      }
      return files;
     }

     /**
      * Extracts text and annotations from Gi2MO RDF/XML also saves the text for indexing into /ideas/gi2moIdeaFile-<ideaURI>.txt
      * @return hashmap (filename, annotations) where filename referes to the created file that contains idea content
      */
     public static HashMap<String,ArrayList<String>> parseGi2moXML(String filename) {

      HashMap<String,ArrayList<String>> files = new HashMap();
      
      try {
      File file = new File(filename);
      DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
      DocumentBuilder db = dbf.newDocumentBuilder();
      Document doc = db.parse(file);
      doc.getDocumentElement().normalize();
      NodeList nodeLst = doc.getElementsByTagName("gi2mo:Idea");

      for (int s = 0; s < nodeLst.getLength(); s++) {
        String fileName =  null;
        String fileText =  null;
        String annotationText = null;
        ArrayList<String> annotations = new ArrayList<String>();

        Node fstNode = nodeLst.item(s);

        if (fstNode.getNodeType() == Node.ELEMENT_NODE) {

          Element fstElmnt = (Element) fstNode;
          String ideaNodeURI = fstElmnt.getAttribute("rdf:about");
          fileName = "ideas/gi2moIdeaFile-"+ideaNodeURI.replaceAll("http://", "").replaceAll("/", "-")+".txt";
          //System.out.println("file: "+fileName);
          //get filename
          NodeList fstNmElmntLst = fstElmnt.getElementsByTagName(gi2moTextProperty);
          Element fstNmElmnt = (Element) fstNmElmntLst.item(0);
          NodeList fstNm = fstNmElmnt.getChildNodes();
          //System.out.println("File Name : "  + ((Node) fstNm.item(0)).getNodeValue());
          fileText = ((Node) fstNm.item(0)).getNodeValue();
          //System.out.println("Idea text: "+fileText);
          //get annotations
          writeFileAsTxt(fileName,fileText);
          NodeList lstNmElmntLst = fstElmnt.getElementsByTagName(gi2moAnnotationProperty);
            for (int k = 0; k < lstNmElmntLst.getLength(); k++) {
              Node annoatationNode = lstNmElmntLst.item(k);
              if (annoatationNode.getNodeType() == Node.ELEMENT_NODE) {
                   Element annotation = (Element) annoatationNode;
                   annotationText=annotation.getAttribute("rdf:resource").replaceAll(gi2moAnnotationPropertyPrefix, "");
                   //System.out.println("annotation: "+annotationText);
                   annotations.add(annotation.getAttribute("rdf:resource").replaceAll(gi2moAnnotationPropertyPrefix, ""));
                   //System.out.println("test:"+ annoatationNode.getTextContent());
              }
            }
        }
        files.put(fileName, annotations);
      }
      } catch (Exception e) {
        e.printStackTrace();
        return null;
      }
      return files;
     }

    private static void printHelp()
    {
            System.out.println("SYNOPSIS");
            System.out.println("goNTogle.jar [-lcamsbkt] document_file ontology_name concept_label1 concept_label_2 ...");
            System.out.println("DESCRIPTION");
            System.out.println("document_file - a .txt file with text to annotate");
            System.out.println("ontology_name - name of the ontology present at the Protege Server, see available ontologies with -l");
            System.out.println("concept_label1 - class label, only used to save annotations for a document");
            System.out.println("");
            System.out.println("Command list:");
            System.out.println("-l	list all ontologies stored on protege server");
            System.out.println("-c	list all concepts for an ontology_name");
            System.out.println("-a	suggest annotations for document");
            System.out.println("-m	same as -a command but automatically saves the suggested values into goNTogle database");
            System.out.println("-s	save document in goNTogle database with specified annotations. Annotations are passed as concept labels.");
            System.out.println("-b	list concepts from a selected ontology that annotate the document in goNTogle database");
            System.out.println("OPTIONAL");
            System.out.println("-k	define k value (optional), by default k=5. This option always must be proceeded with -a (default k-5)");
            System.out.println("-t	threshold for annotation confidence. This can be only used with -m to determine which suggestions should be saved (defaut t=25)");
            System.out.println("-n	indicates that the document_file does not contain text but filenames of files with text. This option is allowed only with -m, -s and -a. The document_file is an xml file with a filename and annotations.");
            System.out.println("-gi2mo	the document_file is treated as RDF/XML file with gi2m0:Ideas inside. The ideas are extracted and turned into documents saved in /ideas directory");
            System.out.println("EXAMPLES:");
            System.out.println("Suggest annotations for a document:");
            System.out.println("  goNTogle.jar -a mytext.txt ACM\\ Ontology");
            System.out.println("Suggest annotations for a document using custom K value:");
            System.out.println("  goNTogle.jar -a mytext.txt ACM\\ Ontology -k 7");
            System.out.println("Save annotations for a document:");
            System.out.println("  goNTogle.jar -s mytext.txt ACM\\ Ontology I.4.3_Enhancement I.3.1_Hardware_Architecture---Graphics_processors");
            System.out.println("List annotations for a document:");
            System.out.println("  goNTogle.jar -b mytext.txt ACM\\ Ontology");
            System.out.println("Suggest annotations for a document using k=7 and save all that show similarity better than 30%:");
            System.out.println("  goNTogle.jar -m mytext.txt ACM\\ Ontology -k 7 -t 30");
            System.out.println("Save annotations for many documents:");
            System.out.println("  goNTogle.jar -s document_list.txt ACM\\ Ontology -n");
            System.out.println("");

    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        boolean gi2moMode = false;
        // TODO code application logic here
        //System.out.println("it works!");
//        String textToAutoAnnotator = "some test text";
//        String fileURL = "/home/cyan/testFile";
//        AutoAnnotatorResults.annotateText(textToAutoAnnotator, fileURL, new MainFrame());


        if (args.length>0)
        {
            if (args[0].compareTo("-l")==0) //list ontologies
            {
                ProtegeInterface interfaceProt = new ProtegeInterface();
                interfaceProt.initProtege();
                interfaceProt.getOntologyList();
                interfaceProt.closeProtegeConnection();
                return;
            }

            if (args[0].compareTo("-c")==0) //list ontology concepts
            {
                if (args.length>1 && args[1].length()>0)
                {
                    ProtegeInterface interfaceProt = new ProtegeInterface();
                    interfaceProt.initProtege();
                    interfaceProt.listOntologyConcepts(args[1]); //"ACM Ontology"
                    interfaceProt.closeProtegeConnection();
                }
                else
                    System.out.println("List ontology concepts: No ontology name specified!");
                return;
            }

            if (args[0].compareTo("-a")==0) //suggest annotations
            {
                OWLModel owlModel;
                if (args.length>1 && args[1].length()>0)
                {
                    if (args.length>2 && args[2].length()>0)
                    {
                        ProtegeInterface interfaceProt = new ProtegeInterface();
                        interfaceProt.initProtege();
                        System.out.println("Loading ontology model...");
                        owlModel = null;
                        owlModel = interfaceProt.getOntology(args[2]); //"ACM Ontology"
                        Ontology.chekOntology(owlModel);
                        interfaceProt.closeProtegeConnection();
                    } else
                    {
                        System.out.println("Missing ontology name!");
                        return;
                    }

                    AutomaticAnnotatorCmd.K=5; //default K value

                    //----checking optional arguments and switches
                    if (args.length>3 && args[3].compareTo("-k")==0) //custom k value
                    {
                        if (args[4].length()>0) //custom k value
                            AutomaticAnnotatorCmd.K=Integer.parseInt(args[4]);
                    }

                    if (args.length>4 && args[4].compareTo("-k")==0) //custom k value (k is 4th param)
                    {
                        if (args[5].length()>0) //custom k value
                            AutomaticAnnotatorCmd.K=Integer.parseInt(args[5]);
                    }

                    Boolean fileListMode=false;
                    if (args.length>3 && args[3].compareTo("-n")==0) //document_file contains file list (in case -n is 3rd param)
                            fileListMode = true;

                    if (args.length>5 && args[5].compareTo("-n")==0) //document_file contains file list (in case -n is 5th param)
                            fileListMode = true;

                    if ((args.length>3 && args[3].length()>0 && args[3].compareToIgnoreCase("-gi2mo")==0) || //testing if goNtoGle is run in Gi2MO compatibility mode
                        (args.length>4 && args[5].length()>0 && args[5].compareToIgnoreCase("-gi2mo")==0))
                        gi2moMode=true;
                    //-------
                    HashMap<String,ArrayList<String>> files = new HashMap();
                    if (gi2moMode)
                    {
                        //parse the document_file to get file list and annotations per file
                        System.out.println("Loading gi2mo file and annotations list...");
                        files = parseGi2moXML(args[1]);
                        if (files==null || files.isEmpty())
                        {
                            System.out.println("No file list found or file list empty!");
                            return;
                        }
                    }
                    else
                    if (fileListMode)
                    {
                        //parse the document_file to get file list and annotations per file
                        System.out.println("Loading file and annotations list...");
                        files = parseFilelist(args[1]);
                        if (files==null || files.isEmpty())
                        {
                            System.out.println("No file list found or file list empty!");
                            return;
                        }
                    }
                    else
                        files.put(args[1] , null);

                    String textToAutoAnnotator = "some test text";
                    String fileURL = "/home/cyan/testFile";

                    Set set = files.entrySet();
                    Iterator filesIterator = set.iterator();

                    System.out.println("<?xml version=\"1.0\"?>");
                    System.out.println("<files>");
                    System.out.println("<kNN_value>"+AutomaticAnnotatorCmd.K+"</kNN_value>");
                    while(filesIterator.hasNext()){
                        Map.Entry filesMap = (Map.Entry)filesIterator.next();

                        //ArrayList<String> classesNames= (ArrayList<String>)filesMap.getValue();
                        String fileName = (String) filesMap.getKey();

                        try {
                            //read text to annotate from file
                            textToAutoAnnotator=readFileAsString(fileName);

                        } catch (IOException ex) {
                            //Logger.getLogger(Annotate.class.getName()).log(Level.SEVERE, null, ex);
                            //System.out.println("Error reading file: "+fileName);
                            continue;
                        }



                        //System.out.println(textToAutoAnnotator);
                            //AutoAnnotatorResults.annotateText(textToAutoAnnotator, fileURL, new MainFrame());

                        System.out.println("<file>");
                        System.out.println("<filename>"+fileName+"</filename>");
                        LinkedHashMap<String, Float> annotatorSuggestions = AutomaticAnnotatorCmd.annotate(textToAutoAnnotator, fileName, owlModel);
                        //System.out.println("Annotation done!");
                        int i = 0;
                        if (annotatorSuggestions!=null)
                        {
                            System.out.println("<annotations>");
                            for (Entry<String, Float> entry : annotatorSuggestions.entrySet()) {
                                if (entry != null) {
                                    System.out.println("<name value=\""+entry.getValue()+"\">"+entry.getKey()+"</name>");
                                    i++;
                                }
                            }
                            System.out.println("</annotations>");
                        } //else
                          //  System.out.println("No annotations found.");

                            //OWLModel
                        System.out.println("</file>");
                    }
                    System.out.println("</files>");
                }
                else
                    System.out.println("Missing name of the file that contains the text to annoate!");
                return;
            }

            if (args[0].compareTo("-s")==0) //save annotations
            {
                OWLModel owlModel = null;
                if (args.length>1 && args[1].length()>0) //if document_name was specified
                {   
                    if (args.length>2 && args[2].length()>0) //extract ontology name
                    {
                        ProtegeInterface interfaceProt = new ProtegeInterface();
                        interfaceProt.initProtege();
                        System.out.println("Loading ontology model...");
                        owlModel = null;
                        owlModel = interfaceProt.getOntology(args[2]); //"ACM Ontology"
                        Ontology.chekOntology(owlModel);
                        interfaceProt.closeProtegeConnection();
                    } else
                    {
                        System.out.println("Missing ontology name!");
                        return;
                    }

                    HashMap<String,ArrayList<String>> files = new HashMap();
                    if ((args.length>3 && args[3].length()>0 && args[3].compareToIgnoreCase("-gi2mo")==0) || //testing if goNtoGle is run in Gi2MO compatibility mode
                        (args.length>4 && args[4].length()>0 && args[4].compareToIgnoreCase("-gi2mo")==0))
                    {

                      files = parseGi2moXML(args[1]); // parse gi2mo file and create txt files with idea text
                        if (files==null || files.isEmpty())
                        {
                            System.out.println("No gi2mo file found or file empty!");
                            return;
                        }
                    } else //regular text file indexing
                    if (args.length>3 && args[3].length()>0 && args[3].compareToIgnoreCase("-n")==0) { //if -n option was activated
                        //parse the document_file to get file list and annotations per file
                        System.out.println("Loading file and annotations list...");
                        String document_list = null;
                        files = parseFilelist(args[1]);
                        if (files==null || files.isEmpty())
                        {
                            System.out.println("No file list found or file list empty!");
                            return;
                        }
                        //System.out.println("done parsing, result: "+files.toString());
                    } else
                    { // if user specified only a single file from the command line
                        ArrayList<String> annotationClassNames= new ArrayList<String>();
                        for (int i=3;i<args.length;i++) //get all annotation class names
                        {
                            annotationClassNames.add(args[i]);
                        }
                        files.put(args[1], annotationClassNames);
                    }

                    //ArrayList<AnnotationObj> annotations;
                    //annotations = Ontology.getInsancesForDoc(owlModel, args[1]);

                    Set set = files.entrySet();
                    Iterator filesIterator = set.iterator();

                    while(filesIterator.hasNext()){
                        Map.Entry filesMap = (Map.Entry)filesIterator.next();

                        ArrayList<String> classesNames= (ArrayList<String>)filesMap.getValue();
                        String fileName = (String) filesMap.getKey();
                        //System.out.println(fileName + " : " + classesNames.toString() );

                        int wordCount = 100;
                        //
                        if (classesNames.size()>0) //if any annotations specified
                        {
                            //load previous annotations - experimental
                            java.util.ArrayList<AnnotationObj> annotations = Ontology.getInsancesForDoc(owlModel, fileName);
                            boolean docAnnot = false;
                            String instanceName = null;
                            int annotPos = 0;
                            for (int i = 0; i < annotations.size(); i++) { //check  if doc have been already annotated
                                if (annotations.get(i).pos == -1) {
                                    docAnnot = true;
                                    annotPos = i;
                                    break;
                                }
                            }
                            if (docAnnot == true) { //an exi gieni xana annot prostheto tis new classes
                                instanceName = annotations.get(annotPos).instanceName;
                                //System.out.println("found prevous annotation instance: "+instanceName);
                                //annotations.get(annotPos).classes.addAll(classesNames);
                                Ontology.addClassesToInstance(owlModel, instanceName, classesNames);
                            }
                            else
                            //set annotations
                            instanceName = Ontology.createInstance(owlModel, fileName, classesNames, -1, wordCount, 100, -1, 0, "Whole Document");
                            //index document
                            IndexInterface indexInterface = new IndexInterface();
                            indexInterface.indexDocument(fileName);
                            System.out.println("Document indexed: " + fileName);
                        } else
                            System.out.println("Error: No annotations specified");
                        //System.out.println("instanceName: "+instanceName);
                    }
                }
            }

            //        annotations = Ontology.getInsancesForDoc(owlModel, fileURL);
            if (args[0].compareTo("-b")==0) //list annotation for a document
            {
                OWLModel owlModel = null;
                if (args.length>1 && args[1].length()>0)
                {
                    if (args.length>2 && args[2].length()>0)
                    {
                        ProtegeInterface interfaceProt = new ProtegeInterface();
                        interfaceProt.initProtege();
                        System.out.println("Loading ontology model...");
                        owlModel = null;
                        owlModel = interfaceProt.getOntology(args[2]); //"ACM Ontology"
                        Ontology.chekOntology(owlModel);
                        interfaceProt.closeProtegeConnection();
                    } else
                    {
                        System.out.println("Missing ontology name!");
                        return;
                    }

                    ArrayList<AnnotationObj> annotations;
                    
                    annotations = Ontology.getInsancesForDoc(owlModel, args[1]);
                    System.out.println("Annotations found: ");
                    for (int i=0;i<annotations.size();i++)
                        for (int j=0;j<annotations.get(i).classes.size();j++)
                            System.out.println(annotations.get(i).classes.get(j));
                }
            }

            if (args[0].compareTo("-m")==0) //suggest annotations and save into database
            {
                OWLModel owlModel;
                if (args.length>1 && args[1].length()>0)
                {
                    if (args.length>2 && args[2].length()>0)
                    {
                        ProtegeInterface interfaceProt = new ProtegeInterface();
                        interfaceProt.initProtege();
                        System.out.println("Loading ontology model...");
                        owlModel = null;
                        owlModel = interfaceProt.getOntology(args[2]); //"ACM Ontology"
                        Ontology.chekOntology(owlModel);
                        interfaceProt.closeProtegeConnection();
                    } else
                    {
                        System.out.println("Missing ontology name!");
                        return;
                    }

                    AutomaticAnnotatorCmd.K=5; //default K value
                    if (args.length>3 && args[3].compareTo("-k")==0) //custom k value (k is 3rd param)
                    {
                        if (args[4].length()>0) //custom k value
                            AutomaticAnnotatorCmd.K=Integer.parseInt(args[4]);
                    }

                    if (args.length>5 && args[5].compareTo("-k")==0) //custom k value (k is 4th param)
                    {
                        if (args[6].length()>0) //custom k value
                            AutomaticAnnotatorCmd.K=Integer.parseInt(args[6]);
                    }


                    if (args.length>3 && args[3].compareTo("-t")==0) //custom threshold value (in case t is 3rd param)
                    {
                        if (args[4].length()>0) //custom k value
                            threshold = Integer.parseInt(args[4]);
                    }
                    
                    if (args.length>5 && args[5].compareTo("-t")==0) //custom threshold value (in case t is 4th param)
                    {
                        if (args[6].length()>0) //custom k value
                            threshold = Integer.parseInt(args[6]);
                    }

                    String textToAutoAnnotator = "some test text";
                    String fileURL = "/home/cyan/testFile";

                    HashMap<String,ArrayList<String>> files = new HashMap();
                    if ((args.length>3 && args[3].length()>0 && args[3].compareToIgnoreCase("-gi2mo")==0) || //testing if goNtoGle is run in Gi2MO compatibility mode
                        (args.length>5 && args[5].length()>0 && args[5].compareToIgnoreCase("-gi2mo")==0) ||
                        (args.length>7 && args[7].length()>0 && args[7].compareToIgnoreCase("-gi2mo")==0))
                    {

                      files = parseGi2moXML(args[1]); // parse gi2mo file and create txt files with idea text
                        if (files==null || files.isEmpty())
                        {
                            System.out.println("No gi2mo file found or file empty!");
                            return;
                        }
                    } else //regular text file indexing
                        files.put(args[1], null);

                    Set set = files.entrySet();
                    Iterator filesIterator = set.iterator();

                    System.out.println("<?xml version=\"1.0\"?>");
                    System.out.println("<files>");
                    System.out.println("<kNN_value>"+AutomaticAnnotatorCmd.K+"</kNN_value>");
                    System.out.println("<treshold_value>"+threshold+"</treshold_value>");
                    while(filesIterator.hasNext()){
                        Map.Entry filesMap = (Map.Entry)filesIterator.next();

                        String fileName = (String) filesMap.getKey();

                        try {
                            //read text to annotate from file
                            textToAutoAnnotator=readFileAsString(fileName);

                        } catch (IOException ex) {
                            Logger.getLogger(Annotate.class.getName()).log(Level.SEVERE, null, ex);
                        }

                        //System.out.println(textToAutoAnnotator);
                            //AutoAnnotatorResults.annotateText(textToAutoAnnotator, fileURL, new MainFrame());
                        //System.out.println("Annotating (k="+AutomaticAnnotatorCmd.K+")...");
                        System.out.println("<file>");
                        System.out.println("<filename>"+fileName+"</filename>");
                        LinkedHashMap<String, Float> annotatorSuggestions = AutomaticAnnotatorCmd.annotate(textToAutoAnnotator, fileName, owlModel);
                        //System.out.println("Searching suggestions done!");

                        ArrayList<String> classesNames= new ArrayList<String>();
                        if (annotatorSuggestions!=null)
                        {
                            System.out.println("<annotations>");
                            for (Entry<String, Float> entry : annotatorSuggestions.entrySet()) {
                                if (entry != null) {
                                    if (entry.getValue()>threshold)
                                    {
                                        classesNames.add(entry.getKey());
                                        //System.out.println(entry.getKey() + "  " + entry.getValue() + " %");
                                        System.out.println("<name value=\""+entry.getValue()+"\">"+entry.getKey()+"</name>");
                                    }
                                }
                            }
                            System.out.println("</annotations>");
                        } else
                        {
                            //System.out.println("No annotations found.");
                            //return;
                            continue;
                        }
                        System.out.println("</file>");

                        int wordCount = 100;
                        //
                        if (classesNames.size()>0) //if any annotations specified
                        {
                            //load previous annotations - experimental
                            //System.out.println("Filtering annotations, threshold="+threshold);
                            java.util.ArrayList<AnnotationObj> annotations = Ontology.getInsancesForDoc(owlModel, fileName);
                            boolean docAnnot = false;
                            String instanceName = null;
                            int annotPos = 0;
                            for (int i = 0; i < annotations.size(); i++) { //check  if doc have been already annotated
                                if (annotations.get(i).pos == -1) {
                                    docAnnot = true;
                                    annotPos = i;
                                    break;
                                }
                            }
                            //System.out.println("Saving annotations and indexing document...");
                            if (docAnnot == true) { //an exi gieni xana annot prostheto tis new classes
                                instanceName = annotations.get(annotPos).instanceName;
                                //System.out.println("found prevous annotation instance: "+instanceName);
                                //annotations.get(annotPos).classes.addAll(classesNames);
                                Ontology.addClassesToInstance(owlModel, instanceName, classesNames);
                            }
                            else
                            //set annotations
                            instanceName = Ontology.createInstance(owlModel, fileName, classesNames, -1, wordCount, 100, -1, 0, "Whole Document");
                            //index document
                            IndexInterface indexInterface = new IndexInterface();
                            indexInterface.indexDocument(fileName);
                        } else
                        {
                            //System.out.println("No annotations meeting specified threshold");
                        }
                        //System.out.println("instanceName: "+instanceName);
                            //OWLModel
                    }
                    System.out.println("</files>");
                }
                else
                    System.out.println("Missing name of the file that contains the text to annoate!");
                return;
            }


            if (args[0].compareTo("-h")==0 || args[0].compareTo("-help")==0 || args[0].compareTo("--h")==0 || args[0].compareTo("--help")==0)
                printHelp();
        }
        else //if no parameter was specified display help
            printHelp();

    }

}
